
## Differences between original game engine and KStarbound

Despite these two pieces of software try to achieve the same
goal of providing environment for mods and their content (including base game,
which is technically a mod), they have different ways of doing so.

While it is no secret that KStarbound contains bits of original code,
whenever be it runtime constants, or json deserialization structures,
they are never copied directly. This file covers most notable differences
between engines which end-users will see.

### Technical differences

* Lighting engine is based off original code, but is heavily modified, such as:
  * Before spreading point lights, potential rectangle is determined, to reduce required calculations
  * Lights are clasterized, and clusters are processed together, **on different threads** (multithreading)
  * Point lights are being spread along **both diagonals**, not only along left-right bottom-top diagonal (can be adjusted using "light quality" setting)
  * While overall performance is marginally better than original game, and scales up to any number of cores, efficiency of spreading algorithm is worse than original
* Chunk rendering is split into render regions, which size can be adjusted in settings
  * Increasing render region size will decrease CPU load when rendering world and increase GPU utilization efficiency, while hurting CPU performance on chunk updates, and vice versa
  * Render region size themselves align with world borders, so 3000x2000 world would have 30x25 sized render regions

### Modding differences
 * Generally, object orientation and parameters can override more properties on fly
   * Space scan of sprite on atlas is not supported yet (original engine does not support this)
 * While original game engine is quite lenient about what it can load, KStarbound aims to be more strict about inputs. This implies KStarbound validates input much more than original engine, while also giving more clear hints at whats wrong with prototypes

### Modding API changes
 * Objects
   * `damageTable` can be defined directly, without referencing other JSON file
