package ru.dbotthepony.kstarbound.defs.world

import com.google.gson.JsonObject
import ru.dbotthepony.kommons.util.Either
import ru.dbotthepony.kommons.vector.Vector2i
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.world.Universe
import ru.dbotthepony.kstarbound.world.UniversePos
import ru.dbotthepony.kstarbound.world.WorldGeometry

class WorldTemplate(val geometry: WorldGeometry) {
	var seed: Long = 0L
		private set
	var worldName: String = ""
		private set
	var worldParameters: VisitableWorldParameters? = null
		private set
	var worldLayout: WorldLayout? = null
		private set
	var skyParameters: SkyParameters = SkyParameters()
		private set
	var celestialParameters: CelestialParameters? = null
		private set

	constructor(worldParameters: VisitableWorldParameters, skyParameters: SkyParameters, seed: Long) : this(WorldGeometry(worldParameters.worldSize, true, false)) {
		this.seed = seed
		this.skyParameters = skyParameters
		this.worldLayout = worldParameters.createLayout(seed)
	}

	fun determineName() {
		if (celestialParameters != null) {
			worldName = celestialParameters!!.name
		} else if (worldParameters is FloatingDungeonWorldParameters) {

		} else {
			worldName = ""
		}
	}

	@JsonFactory
	data class SerializedForm(
		val celestialParameters: CelestialParameters? = null,
		val worldParameters: VisitableWorldParameters? = null,
		val skyParameters: SkyParameters = SkyParameters(),
		val seed: Long = 0L,
		val size: Either<WorldGeometry, Vector2i>,
		val regionData: WorldLayout? = null,
		//val customTerrainRegions:
	)

	fun toJson(): JsonObject {
		val data = Starbound.gson.toJsonTree(SerializedForm(
			celestialParameters, worldParameters, skyParameters, seed,
			if (Starbound.IS_LEGACY_JSON) Either.right(geometry.size) else Either.left(geometry),
			worldLayout
		)) as JsonObject

		return data
	}

	companion object {
		suspend fun create(coordinate: UniversePos, universe: Universe): WorldTemplate {
			val params = universe.parameters(coordinate) ?: throw IllegalArgumentException("$universe has nothing at $coordinate!")
			val visitable = params.visitableParameters ?: throw IllegalArgumentException("$coordinate of $universe is not visitable")

			val template = WorldTemplate(WorldGeometry(visitable.worldSize, true, false))

			template.seed = params.seed
			template.worldLayout = visitable.createLayout(params.seed)
			template.skyParameters = SkyParameters.create(coordinate, universe)
			template.celestialParameters = params
			template.worldParameters = visitable

			template.determineName()

			return template
		}

		fun fromJson(data: JsonObject): WorldTemplate {
			val load = Starbound.gson.fromJson(data, SerializedForm::class.java)
			val template = WorldTemplate(load.size.map({ it }, { WorldGeometry(it, true, false) }))

			template.celestialParameters = load.celestialParameters
			template.worldParameters = load.worldParameters
			template.skyParameters = load.skyParameters
			template.seed = load.seed
			template.worldLayout = load.regionData

			template.determineName()

			return template
		}
	}
}
