package ru.dbotthepony.kstarbound.world.api

import com.github.benmanes.caffeine.cache.Interner
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.network.LegacyNetworkCellState
import ru.dbotthepony.kstarbound.util.HashTableInterner
import ru.dbotthepony.kstarbound.world.physics.CollisionType
import java.io.DataInputStream
import java.io.DataOutputStream

sealed class AbstractCell {
	abstract val foreground: AbstractTileState
	abstract val background: AbstractTileState
	abstract val liquid: AbstractLiquidState

	abstract val dungeonId: Int
	abstract val biome: Int
	abstract val envBiome: Int
	abstract val isIndestructible: Boolean

	abstract fun immutable(): ImmutableCell
	abstract fun mutable(): MutableCell

	open fun toLegacyNet(): LegacyNetworkCellState {
		return LegacyNetworkCellState(background.toLegacyNet(), foreground.toLegacyNet(), foreground.material.value.collisionKind, biome, envBiome, liquid.toLegacyNet(), dungeonId)
	}

	abstract fun tile(background: Boolean): AbstractTileState

	fun write(stream: DataOutputStream) {
		foreground.write(stream)
		background.write(stream)
		liquid.write(stream)

		stream.write(0) // collisionMap

		stream.writeShort(dungeonId)
		stream.writeByte(biome)
		stream.writeByte(envBiome)
		stream.writeBoolean(isIndestructible)

		stream.write(0) // unknown
	}

	companion object {
		fun skip(stream: DataInputStream) {
			AbstractTileState.skip(stream)
			AbstractTileState.skip(stream)
			AbstractLiquidState.skip(stream)
			stream.skipNBytes(1 + 2 + 1 + 1 + 1 + 1)
		}

		@JvmStatic
		protected val POOL: Interner<ImmutableCell> = if (Starbound.DEDUP_CELL_STATES) Starbound.interner() else Interner { it }

		val EMPTY: ImmutableCell = POOL.intern(ImmutableCell(AbstractTileState.EMPTY, AbstractTileState.EMPTY, AbstractLiquidState.EMPTY, 0, 0, 0, false))
		val NULL: ImmutableCell = POOL.intern(ImmutableCell(AbstractTileState.NULL, AbstractTileState.NULL, AbstractLiquidState.EMPTY, 0, 0, 0, false))
	}
}
